Package: NOISeq
Type: Package
Title: Exploratory analysis and differential expression for RNA-seq
        data
Version: 2.56.0
Date: 2014-02-24
Author: Sonia Tarazona, Pedro Furio-Tari, Maria Jose Nueda, Alberto Ferrer and Ana Conesa
Maintainer: Sonia Tarazona <sotacam@eio.upv.es>
Depends: R (>= 2.13.0), methods, Biobase (>= 2.13.11), splines (>=
        3.0.1), Matrix (>= 1.2)
Description: Analysis of RNA-seq expression data or other similar kind
        of data. Exploratory plots to evualuate saturation, count
        distribution, expression per chromosome, type of detected
        features, features length, etc. Differential expression between
        two experimental conditions with no parametric assumptions.
License: Artistic-2.0
LazyLoad: yes
biocViews: ImmunoOncology, RNASeq, DifferentialExpression,
        Visualization, Sequencing
git_url: https://git.bioconductor.org/packages/NOISeq
git_branch: RELEASE_3_23
git_last_commit: a85acf6
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:00:10 UTC; biocbuild
Built: R 4.6.0; ; 2026-04-29 04:53:03 UTC; unix
